import os

import click

import naas_abi_cli
from naas_abi_cli.cli.utils.Copier import Copier

from .new import new
from .utils import to_pascal_case


@new.command("agent")
@click.argument("agent_name", required=True)
@click.argument("agent_path", required=False, default=".")
def _new_agent(agent_name: str, agent_path: str = "."):
    new_agent(agent_name, agent_path)


def new_agent(
    agent_name: str,
    agent_path: str = ".",
    extra_values: dict = {},
):
    agent_name = to_pascal_case(agent_name)

    if agent_path == ".":
        agent_path = os.getcwd()

    if not os.path.exists(agent_path):
        os.makedirs(agent_path, exist_ok=True)

    copier = Copier(
        templates_path=os.path.join(
            os.path.dirname(naas_abi_cli.__file__), "cli/new/templates/agent"
        ),
        destination_path=agent_path,
    )

    if "module_name_snake" not in extra_values:
        extra_values["module_name_snake"] = "your_module_name"

    copier.copy(values={"agent_name_pascal": to_pascal_case(agent_name)} | extra_values)



from dataclasses import dataclass
from enum import Enum
from typing import Annotated, Optional

from langchain_core.tools import BaseTool, StructuredTool
from naas_abi_core.services.triple_store.TripleStorePorts import ITripleStoreService
from naas_abi_core.utils.Expose import APIRouter
from naas_abi_core.workflow import Workflow, WorkflowConfiguration
from naas_abi_core.workflow.workflow import WorkflowParameters
from pydantic import Field

@dataclass
class {{workflow_name_pascal}}WorkflowConfiguration(WorkflowConfiguration):
    """Configuration for {{workflow_name_pascal}} workflow."""


    pass

class {{workflow_name_pascal}}WorkflowParameters(WorkflowParameters):
    """Parameters for {{workflow_name_pascal}} workflow."""

    pass

    # example_parameter: Annotated[
    #     str,
    #     Field(
    #         ...,
    #         description="Description of the example parameter",
    #         example="Example value",
    #     ),
    # ]


class {{workflow_name_pascal}}Workflow(Workflow[{{workflow_name_pascal}}WorkflowParameters]):
    """Workflow for {{workflow_name_pascal}}."""

    __configuration: {{workflow_name_pascal}}WorkflowConfiguration

    def __init__(self, configuration: {{workflow_name_pascal}}WorkflowConfiguration):
        super().__init__(configuration)
        self.__configuration = configuration

    def run(
        self, parameters: {{workflow_name_pascal}}WorkflowParameters
    ) -> dict | list[dict]:
        # Implement the workflow logic here
        pass


    def as_tools(self) -> list[BaseTool]:
        return [
            StructuredTool(
                name="{{workflow_name_pascal}}",
                description="Description of what the workflow does",
                func=lambda **kwargs: self.run(
                    {{workflow_name_pascal}}WorkflowParameters(**kwargs)
                ),
                args_schema={{workflow_name_pascal}}WorkflowParameters,
            )
        ]

    def as_api(
        self,
        router: APIRouter,
        route_name: str = "",
        name: str = "",
        description: str = "",
        description_stream: str = "",
        tags: list[str | Enum] | None = None,
    ) -> None:
        if tags is None:
            tags = []
        return None



import os
import subprocess
import sys

import click

from .agent import agent
from .chat import chat
from .config import config
from .deploy import deploy
from .init import init
from .module import module
from .new import new
from .run import run
from .secret import secrets
from .setup import setup


@click.group("abi")
def _main():
    pass


_main.add_command(secrets)
_main.add_command(config)
_main.add_command(module)
_main.add_command(agent)
_main.add_command(chat)
_main.add_command(new)
_main.add_command(init)
_main.add_command(deploy)
_main.add_command(run)
_main.add_command(setup)
ran = False


def main():
    global ran
    if ran:
        return
    ran = True

    # Check how the project is being runned.
    if os.getenv("LOCAL_UV_RAN") is None:
        if "pyproject.toml" in os.listdir(os.getcwd()):
            with open("pyproject.toml", "r") as file:
                if "naas-abi-cli" in file.read():
                    arguments = (
                        "uv run --active python -m naas_abi_cli.cli".split(" ")
                        + sys.argv[1:]
                    )
                    try:
                        subprocess.run(
                            arguments,
                            cwd=os.getcwd(),
                            env={**os.environ, "LOCAL_UV_RAN": "true"},
                            check=True,
                        )
                    except Exception:
                        pass

                    return
    _main()


main()
main()



import click
from naas_abi_core import logger


@click.command("chat")
@click.argument("module-name", type=str, default="")
@click.argument("agent-name", type=str, default="")
def chat(module_name: str = "", agent_name: str = ""):
    from naas_abi_core.engine.Engine import Engine

    engine = Engine()

    if module_name == "" and agent_name == "":
        module_name, agent_name = engine.configuration.default_agent.split(" ")

    engine.load(module_names=[module_name])

    from naas_abi_core.apps.terminal_agent.main import run_agent

    if module_name not in engine.modules:
        raise ValueError(f"Module {module_name} not found")

    logger.debug(f"Module agents: {engine.modules[module_name].agents}")

    for agent_class in engine.modules[module_name].agents:
        logger.debug(f"Agent class: {agent_class.__name__}")
        if agent_class.__name__ == agent_name:
            run_agent(agent_class.New())
            break



import dagster as dg
from naas_abi_core.orchestrations.DagsterOrchestration import DagsterOrchestration


class {{orchestration_name_pascal}}Orchestration(DagsterOrchestration):

    @classmethod
    def New(cls) -> "{{orchestration_name_pascal}}Orchestration":
        return cls(
            definitions=dg.Definitions(
                assets=[],
                schedules=[],
                jobs=[],
                sensors=[],
            )
        )


import os
import shutil
import subprocess
from uuid import uuid4

import click
import requests
from naas_abi_core import logger
from naas_abi_core.engine.engine_configuration.EngineConfiguration import (
    EngineConfiguration,
)
from pydantic import BaseModel
from rich.console import Console
from rich.markdown import Markdown


@click.group("deploy")
def deploy():
    pass


class Container(BaseModel):
    name: str
    image: str
    port: int
    cpu: str
    memory: str
    env: dict


class Space(BaseModel):
    name: str
    containers: list[Container]


class NaasAPIClient:
    naas_api_key: str
    base_url: str

    def __init__(self, naas_api_key: str):
        self.naas_api_key = naas_api_key
        self.base_url = "https://api.naas.ai"

    def create_registry(self, name: str):
        response = requests.post(
            f"{self.base_url}/registry/",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
            json={"name": name},
        )
        if response.status_code == 409:
            return self.get_registry(name)
        response.raise_for_status()
        return response.json()

    def get_registry(self, name: str):
        response = requests.get(
            f"{self.base_url}/registry/{name}",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
        )
        response.raise_for_status()
        return response.json()

    def get_registry_credentials(self, name: str):
        response = requests.get(
            f"{self.base_url}/registry/{name}/credentials",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
        )
        response.raise_for_status()
        return response.json()

    def update_space(self, space: Space) -> dict:
        response = requests.put(
            f"{self.base_url}/space/{space.name}",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
            json=space.model_dump(),
        )
        response.raise_for_status()
        return response.json()

    def create_space(self, space: Space) -> dict:
        response = requests.post(
            f"{self.base_url}/space/",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
            json=space.model_dump(),
        )

        if response.status_code == 409:
            return self.update_space(space)
        elif response.status_code == 402:
            raise click.ClickException(
                "You must have an active subscription to create a space on naas.ai."
            )

        response.raise_for_status()
        return response.json()

    def get_space(self, name: str) -> dict:
        response = requests.get(
            f"{self.base_url}/space/{name}",
            headers={"Authorization": f"Bearer {self.naas_api_key}"},
        )
        response.raise_for_status()
        return response.json()


class NaasDeployer:
    image_name: str

    naas_api_client: NaasAPIClient

    def __init__(self, configuration: EngineConfiguration):
        self.configuration = configuration
        self.image_name = str(uuid4())
        if configuration.deploy is None:
            # Fail fast with a clear, user-facing error instead of an assertion.
            raise click.ClickException(
                "Deploy configuration is missing; please add a deploy section before running this command."
            )
        self.naas_api_client = NaasAPIClient(configuration.deploy.naas_api_key)

    def docker_build(self, image_name: str):
        subprocess.run(
            f"docker build -t {image_name} . --platform linux/amd64", shell=True
        )

    def deploy(self):
        assert self.configuration.deploy is not None
        registry = self.naas_api_client.create_registry(
            self.configuration.deploy.space_name
        )

        uid = str(uuid4())

        image_name = f"{registry['registry']['uri']}:{uid}"
        self.docker_build(image_name)
        credentials = self.naas_api_client.get_registry_credentials(
            self.configuration.deploy.space_name
        )
        docker_login_command = f"docker login -u {credentials['credentials']['username']} -p {credentials['credentials']['password']} {registry['registry']['uri']}"
        subprocess.run(docker_login_command, shell=True)
        subprocess.run(f"docker push {image_name}", shell=True)

        image_sha = (
            subprocess.run(
                "docker inspect --format='{{index .RepoDigests 0}}' "
                + image_name
                + " | cut -d'@' -f2",
                shell=True,
                capture_output=True,
            )
            .stdout.strip()
            .decode("utf-8")
        )

        if image_sha is None or image_sha == "":
            raise click.ClickException(
                "Failed to get image SHA. Please check if the image is correctly built and pushed to the registry."
            )

        image_name_with_sha = f"{image_name.replace(':' + uid, '')}@{image_sha}"

        self.naas_api_client.create_space(
            Space(
                name=self.configuration.deploy.space_name,
                containers=[
                    Container(
                        name="api",
                        image=image_name_with_sha,
                        port=9879,
                        cpu="1",
                        memory="1Gi",
                        env=self.configuration.deploy.env,
                    )
                ],
            )
        )

        self.naas_api_client.get_space(self.configuration.deploy.space_name)

        Console().print(
            Markdown(f"""
# Deployment successful

- Space: {self.configuration.deploy.space_name}
- Image: {image_name_with_sha}
- URL: https://{self.configuration.deploy.space_name}.default.space.naas.ai

                                 """)
        )


def _get_configuration(env: str):
    # Set the ENV environment variable for the EngineConfiguration.load_configuration() to load the correct configuration file.
    os.environ["ENV"] = env

    configuration: EngineConfiguration = EngineConfiguration.load_configuration()

    if configuration.deploy is None:
        logger.error(
            "Deploy configuration not found in the yaml configuration file. Please add a deploy section to the configuration file."
        )
        raise click.ClickException("Missing deploy configuration; aborting.")

    return configuration


@deploy.command("naas")
@click.option(
    "-e",
    "--env",
    type=str,
    default="prod",
    help="Environment to use (default: prod). This is used to know which configuration file to load. (config.prod.yaml, config.yaml, ...)",
)
def naas(env: str):

    configuration = _get_configuration(env)

    deployer = NaasDeployer(configuration)
    deployer.deploy()


@deploy.command("local")
@click.option(
    "-e",
    "--env",
    type=str,
    default="local",
    help="Environment to use (default: local). This is used to know which configuration file to load. (config.local.yaml, config.yaml, ...)",
)
def local_deploy(env: str):

    if not os.path.exists(os.path.join(os.getcwd(), ".deploy")):
        import naas_abi_cli

        from ..utils.Copier import Copier

        # Create .deploy folder if it doesn't exist
        os.makedirs(os.path.join(os.getcwd(), ".deploy"), exist_ok=False)

        copier = Copier(
            templates_path=os.path.join(
                os.path.dirname(naas_abi_cli.__file__), "cli/deploy/templates/local"
            ),
            destination_path=os.path.join(os.getcwd(), ".deploy"),
        )

        copier.copy(
            values={
                "POSTGRES_USER": "abi",
                "POSTGRES_PASSWORD": str(uuid4()),
                "POSTGRES_DB": "abi",
                "MINIO_ROOT_PASSWORD": str(uuid4()),
                "RABBITMQ_PASSWORD": str(uuid4()),
            }
        )

        shutil.move(
            os.path.join(os.getcwd(), ".deploy", "docker-compose.yml"),
            os.path.join(os.getcwd(), "docker-compose.yml"),
        )

        # Concat .deploy/.env into .env
        with open(os.path.join(os.getcwd(), ".deploy", ".env"), "r") as f:
            with open(os.path.join(os.getcwd(), ".env"), "a") as f2:
                f2.write(f.read())
        # Remove .deploy/.env
        os.remove(os.path.join(os.getcwd(), ".deploy", ".env"))


@deploy.command("local-up")
@click.option(
    "-d",
    "--detach",
    is_flag=True,
    default=False,
    help="Run containers in the background.",
)
def local_up(detach: bool):
    command = (
        "docker compose --file docker-compose.yml --env-file .env --profile infrastructure --profile container up"
        + (" -d" if detach else "")
    )
    print(command)
    subprocess.run(command, shell=True)


@deploy.command("local-down")
@click.option(
    "-v",
    "--volumes",
    is_flag=True,
    default=False,
    help="Remove volumes along with the containers.",
)
def local_down(volumes: bool):
    subprocess.run(
        "docker compose --file docker-compose.yml --env-file .env --profile infrastructure --profile container down"
        + (" -v" if volumes else ""),
        shell=True,
    )


@deploy.command("local-logs")
def local_logs():
    subprocess.run(
        "docker compose --file docker-compose.yml --env-file .env --profile infrastructure --profile container logs -f",
        shell=True,
    )



import os

import click
import naas_abi_cli
from naas_abi_cli.cli.utils.Copier import Copier

from .new import new
from .utils import to_pascal_case


@new.command("orchestration")
@click.argument("orchestration_name", required=True)
@click.argument("orchestration_path", required=False, default=".")
def _new_orchestration(orchestration_name: str, orchestration_path: str = "."):
    new_orchestration(orchestration_name, orchestration_path)


def new_orchestration(orchestration_name: str, orchestration_path: str = ".", extra_values: dict = {}):
    orchestration_name = to_pascal_case(orchestration_name)

    if orchestration_path == ".":
        orchestration_path = os.getcwd()

    if not os.path.exists(orchestration_path):
        os.makedirs(orchestration_path, exist_ok=True)

    copier = Copier(
        templates_path=os.path.join(
            os.path.dirname(naas_abi_cli.__file__), "cli/new/templates/orchestration"
        ),
        destination_path=orchestration_path,
    )

    copier.copy(
        values={"orchestration_name_pascal": to_pascal_case(orchestration_name)} | extra_values
    )
